<?php

namespace WcDPD;

defined('ABSPATH') || exit;

/**
 * Shipping class
 */
class Shipping
{
    public const SESSION_CHOSEN_PARCELSHOP_KEY = 'wc_dpd_chosen_parcelshop';

    public static function init()
    {
        add_filter('woocommerce_shipping_methods', [__CLASS__, 'registerShippingMethodsAndSettings'], 10, 1);
        add_filter('woocommerce_after_shipping_rate', [__CLASS__, 'displayDpdParcelShopShippingMethodAdditionalContent'], 10, 2);
        add_action('wp_footer', [__CLASS__, 'addParcelShopPopup'], 10, 2);
        add_action('woocommerce_checkout_process', [__CLASS__, 'validateParcelShopShippingMethodFields'], 10, 1);
    }

    /**
     * Register dpd export settings
     *
     * @return array
     */
    public static function registerShippingMethodsAndSettings($methods)
    {
        $methods[DpdExportSettings::SETTINGS_ID_KEY] = new DpdExportSettings();
        $methods[DpdParcelShopShippingMethod::SETTINGS_ID_KEY] = new DpdParcelShopShippingMethod();

        return $methods;
    }

    /**
     * Validate parcelshop shipping method required fields
     *
     * @return void
     */
    public static function validateParcelShopShippingMethodFields()
    {
        if (empty($_POST['shipping_method'][0])) {
            return;
        }

        if ($_POST['shipping_method'][0] != DpdParcelShopShippingMethod::SETTINGS_ID_KEY) {
            return;
        }

        $parcelshop_name = !empty($_POST[DpdParcelShopShippingMethod::PARCELSHOP_NAME_META_KEY]) ? (string) wp_kses_post($_POST[DpdParcelShopShippingMethod::PARCELSHOP_NAME_META_KEY]) : '';
        $parcelshop_street = !empty($_POST[DpdParcelShopShippingMethod::PARCELSHOP_STREET_META_KEY]) ? (string) wp_kses_post($_POST[DpdParcelShopShippingMethod::PARCELSHOP_STREET_META_KEY]) : '';
        $parcelshop_zip = !empty($_POST[DpdParcelShopShippingMethod::PARCELSHOP_ZIP_META_KEY]) ? (string) wp_kses_post($_POST[DpdParcelShopShippingMethod::PARCELSHOP_ZIP_META_KEY]) : '';
        $parcelshop_city = !empty($_POST[DpdParcelShopShippingMethod::PARCELSHOP_CITY_META_KEY]) ? (string) wp_kses_post($_POST[DpdParcelShopShippingMethod::PARCELSHOP_CITY_META_KEY]) : '';
        $parcelshop_country_code = !empty($_POST[DpdParcelShopShippingMethod::PARCELSHOP_COUNTRY_CODE_META_KEY]) ? (string) wp_kses_post($_POST[DpdParcelShopShippingMethod::PARCELSHOP_COUNTRY_CODE_META_KEY]) : '';

        if (
            ! $parcelshop_name ||
            ! $parcelshop_street ||
            ! $parcelshop_zip ||
            ! $parcelshop_city ||
            ! $parcelshop_country_code
        ) {
            wc_add_notice(__("You have to choose a parcelshop.", "wc-dpd"), 'error');
        }
    }

    /**
     * Display parcelshop shipping method additional content
     *
     * @param object $method
     * @param int $index
     *
     * @return object
     */
    public static function displayDpdParcelShopShippingMethodAdditionalContent($method, $index)
    {
        if (is_admin()) {
            return;
        }

        if (!isset($method->id)) {
            return $method;
        }

        if ($method->id !== DpdParcelShopShippingMethod::SETTINGS_ID_KEY) {
            return $method;
        }

        if (
            !isset(WC()->session) ||
            !isset(WC()->session->chosen_shipping_methods) ||
            empty(WC()->session->chosen_shipping_methods[0])
        ) {
            return $method;
        }

        $chosen_shipping_method = wp_kses_post(WC()->session->chosen_shipping_methods[0]);
        if ($chosen_shipping_method !== DpdParcelShopShippingMethod::SETTINGS_ID_KEY) {
            return $method;
        }

        $countries = (array) WC()->countries->get_allowed_countries();
        $chosen_parcelshop_data = WC()->session->get(self::SESSION_CHOSEN_PARCELSHOP_KEY);

        $chosen_parcelshop_id = !empty($chosen_parcelshop_data[DpdParcelShopShippingMethod::PARCELSHOP_ID_META_KEY]) ? (int) wp_kses_post($chosen_parcelshop_data[DpdParcelShopShippingMethod::PARCELSHOP_ID_META_KEY]) : '';
        $chosen_parcelshop_name = !empty($chosen_parcelshop_data[DpdParcelShopShippingMethod::PARCELSHOP_NAME_META_KEY]) ? (string) wp_kses_post($chosen_parcelshop_data[DpdParcelShopShippingMethod::PARCELSHOP_NAME_META_KEY]) : '';
        $chosen_parcelshop_street = !empty($chosen_parcelshop_data[DpdParcelShopShippingMethod::PARCELSHOP_STREET_META_KEY]) ? (string) wp_kses_post($chosen_parcelshop_data[DpdParcelShopShippingMethod::PARCELSHOP_STREET_META_KEY]) : '';
        $chosen_parcelshop_zip = !empty($chosen_parcelshop_data[DpdParcelShopShippingMethod::PARCELSHOP_ZIP_META_KEY]) ? (string) wp_kses_post($chosen_parcelshop_data[DpdParcelShopShippingMethod::PARCELSHOP_ZIP_META_KEY]) : '';
        $chosen_parcelshop_city = !empty($chosen_parcelshop_data[DpdParcelShopShippingMethod::PARCELSHOP_CITY_META_KEY]) ? (string) wp_kses_post($chosen_parcelshop_data[DpdParcelShopShippingMethod::PARCELSHOP_CITY_META_KEY]) : '';
        $chosen_parcelshop_zip_and_city = $chosen_parcelshop_zip;
        $chosen_parcelshop_zip_and_city .= $chosen_parcelshop_zip_and_city ? ' ' . $chosen_parcelshop_city : $chosen_parcelshop_city;

        $chosen_parcelshop_country_code = !empty($chosen_parcelshop_data[DpdParcelShopShippingMethod::PARCELSHOP_COUNTRY_CODE_META_KEY]) ? (string) wp_kses_post($chosen_parcelshop_data[DpdParcelShopShippingMethod::PARCELSHOP_COUNTRY_CODE_META_KEY]) : '';
        $chosen_parcelshop_country_name = isset($countries[strtoupper($chosen_parcelshop_country_code)]) ? $countries[strtoupper($chosen_parcelshop_country_code)] : '';

        $chosen_parcelshop_text = esc_html(implode(', ', array_filter([$chosen_parcelshop_name, $chosen_parcelshop_street, $chosen_parcelshop_zip_and_city, $chosen_parcelshop_country_name ])));

        echo include_template('parcelshop-shipping-method-content.php', [
            'chosen_parcelshop_id' =>  $chosen_parcelshop_id,
            'chosen_parcelshop_name' =>  $chosen_parcelshop_name,
            'chosen_parcelshop_street' =>  $chosen_parcelshop_street,
            'chosen_parcelshop_city' =>  $chosen_parcelshop_city,
            'chosen_parcelshop_zip' =>  $chosen_parcelshop_zip,
            'chosen_parcelshop_country_code' =>  $chosen_parcelshop_country_code,
            'chosen_parcelshop_text' =>  $chosen_parcelshop_text,
        ]);

        return $method;
    }

    /**
     * Add parcelshop shipping method popup html code
     *
     * @return void
     */
    public static function addParcelShopPopup()
    {
        if (is_admin()) {
            return;
        }

        if (!is_cart() && !is_checkout()) {
            return;
        }

        // if (!did_action('woocommerce_load_shipping_methods')) {
        //     return;
        // }

        // Include popup html
        echo include_template('parcelshop-popup.php', [
            'countries' => (array) WC()->countries->get_allowed_countries(),
            'base_country_code' =>  (string) WC()->countries->get_base_country()
        ]);
    }
}
